package ch.std.book.repositories;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import ch.std.book.jpa.Author;
import ch.std.book.jpa.Book;
import jakarta.persistence.EntityManager;

@ExtendWith(SpringExtension.class)
@DataJpaTest
@ActiveProfiles("unittest")
public class BookAuthorRepositoryJPATest {

	@Autowired
	private BookRepository bookRepository;
	@Autowired
	private AuthorRepository authorRepository;
	@Autowired
	private EntityManager entityManager;

	private Book book;
	private Author author;

	@BeforeEach
	public void setup() {
		book = new Book("978-1617292545", "this is the title", "that's the description", "my publisher");
		author = new Author("my author");
		author.getBookList().add(book);
		book.getAuthorList().add(author);
		this.bookRepository.save(book);
		this.authorRepository.save(author);
		this.entityManager.flush();
		this.entityManager.clear(); // clear jpa session
	}

	@Test
	public void testFindByAuthorName() {
		List<Book> foundBookList = this.bookRepository.findByAuthorList_Name("my author");
		assertNotNull(foundBookList);
		assertEquals(1, foundBookList.size());
	}

}
